/*
	$Author: me $ - $Date: 1999/04/10 20:36:15 $
*/
#include <Be.h>
#include <NetProtocol.h>

// NOTE: be sure to set the packets base to the beginning of
// the protocol's header.

#include "ip.h"
#include "tcp.h"
#include "ethernet.h"
#include "debug.h"

void dump_ethernet_packet(FILE* f, BNetPacket* pkt)
{
	ethernet_header eth_h;
	pkt->Read(0x0,(char*)&eth_h, sizeof(eth_h));

	fprintf(f, "dst addr: [");
	for(int i = 0; i < ETHERNET_ADDR_LEN; i++)
		fprintf(f,"%02x%s", eth_h.dst.octet[i],(i == ETHERNET_ADDR_LEN -1) ? "]\n" : ":");
	fprintf(f, "src addr: [");
	for(int i = 0; i < ETHERNET_ADDR_LEN; i++)
		fprintf(f,"%02x%s", eth_h.src.octet[i],(i == ETHERNET_ADDR_LEN -1) ? "]\n" : ":");;
	fprintf(f, "type: %d\n", eth_h.type);
}

void dump_ip_packet(FILE* f, BNetPacket* pkt)
{
	uint8* addr;
	ip_header ip_h;
	pkt->Read(0x0,(char*)&ip_h, sizeof(ip_h));

	fprintf(f, "version: %d\n", ip_h.version);
	fprintf(f, "header length: %d\n", ip_h.header_length);
	fprintf(f, "type of service: %d\n", ip_h.type);
	fprintf(f, "total length: %d\n", ip_h.length);
	fprintf(f, "identification: %d\n", ip_h.id);
	fprintf(f, "fragment offset field: %d\n", ip_h.offset);
	fprintf(f, "time to live: %d\n", ip_h.ttl);
	fprintf(f, "protocol: %d\n", ip_h.protocol);
	fprintf(f, "checksum: %d\n", ip_h.checksum);
	fprintf(f, "source ip address: [");
	addr = (uint8*)&ip_h.src;
	for(int i = 0; i < 4; i++)
		fprintf(f, "%d%s", addr[i], (i == 3) ? "]\n" : ".");
	addr = (uint8*)&ip_h.dst;
	fprintf(f, "destination ip address: [");
	for(int i = 0; i < 4; i++)
		fprintf(f, "%d%s", addr[i], (i == 3) ? "]\n" : ".");
}

void dump_tcp_packet(FILE* f, BNetPacket* pkt)
{
	tcp_header tcp_h;
	pkt->Read(0x0,(char*)&tcp_h, sizeof(tcp_h));

	fprintf(f, "source port: %d\n", tcp_h.src_port);
	fprintf(f, "destination port: %d\n", tcp_h.dst_port);
	fprintf(f, "sequence number: %d\n", tcp_h.seq_num);
	fprintf(f, "acknowledgement number: %d\n", tcp_h.ack_num);
	fprintf(f, "offset: %d\n", tcp_h.offset);
	fprintf(f, "reserved: %d\n", tcp_h.reserved);
	fprintf(f, "flags: %d\n", tcp_h.flags);
	fprintf(f, "window: %d\n", tcp_h.window);
	fprintf(f, "checksum: %d\n", tcp_h.checksum);
	fprintf(f, "urgent pointer: %d\n", tcp_h.urgent);
}

void dump_raw_packet(FILE* f, BNetPacket* pkt)
{
	int pkt_size = pkt->Size();
	
	for (int i = 0; i < pkt_size; i++) 
	{
	    fprintf(f, "%s%02x", (i % 0x10) == 0 ? "\n" : " ",
				(uchar) (pkt->Data())[i]);
	}
    fprintf(f, "\n"); fflush(f);
}

void dump_port_map(FILE* f, map<pair<uint32, uint16>, uint16>& port_map)
{
	for(map<pair<uint32, uint16>, uint16>::iterator it = port_map.begin();
			it != port_map.end(); it++)
	{
		dump_map(f,(*it).second,(*it).first.first,(*it).first.second);
	}
	fflush(f);
}

void dump_reverse_port_map(FILE* f, map<uint16, pair<uint32, uint16> >& port_map)
{
	for(map<uint16, pair<uint32, uint16> >::iterator it = port_map.begin();
			it != port_map.end(); it++)
	{
		dump_map(f,(*it).first,(*it).second.first,(*it).second.second);
	}
	fflush(f);
}

void dump_port_socket_map(FILE* f, map<uint16, pair<int, bigtime_t> >& pm_map)
{
	for(map<uint16, pair<int, bigtime_t> > ::iterator it = pm_map.begin();
			it != pm_map.end(); it++)
	{
		fprintf(f,"%hu : %d : %Ld\n", (*it).first, (*it).second.first, (*it).second.second);
	}
	fflush(f);
}

void dump_socket_pool(FILE* f, list<pair<int, uint16> >& socket_pool)
{
	for(list<pair<int, uint16> > ::iterator it = socket_pool.begin();
			it != socket_pool.end(); it++)
	{
		fprintf(f,"%hu : %d\n", (*it).second, (*it).first);
	}	
	fflush(f);
}

void dump_map(FILE* f, uint16 map_port, uint32 ip, uint16 real_port)
{
	in_addr addr;
	addr.s_addr = ip;

	fprintf(f,"%hu -> %s:%hu\n",map_port,ip,real_port);
}