#ifndef _ICMP_H
#define _ICMP_H
/*
	$Author: me $ - $Date: 1999/03/21 04:04:01 $
*/
/*----------------------------------------------------------------------------------------
	NAT
------------------------------------------------------------------------------------------*/
// Network Address Translation for the BeOS
// Richard Burgess <rickb@spy.net>

#include "ip.h"

/*----------------------------------------------------------------------------------------
	ICMP Format
------------------------------------------------------------------------------------------*/
// RFC 792
// The ICMP Packet header is that same as the IP Header

// ICMP Packet Offsets
#define ICMP_TYPE_OFFSET		0
#define ICMP_CODE_OFFSET		1
#define ICMP_CHECKSUM_OFFSET	2
#define ICMP_IDENT_OFFSET		4
#define ICMP_SEQ_NUM_OFFSET		6
#define ICMP_IP_HEADER_OFFSET	8

// ICMP Message Types
#define	ICMP_ECHO_REPLY			0
#define ICMP_DST_UNREACHABLE	3
	#define	ICMP_NET_UNREACHABLE	0
	#define ICMP_HOST_UNREACHABLE	1
	#define ICMP_PROTO_UNREACHABLE	2
	#define ICMP_PORT_UNREACHABLE	3
	#define ICMP_FRAGMENT			4
	#define SRC_ROUTE_FAILED		5
#define ICMP_SRC_QUENCH			4
#define ICMP_REDIRECT			5
#define ICMP_ECHO				8
#define ICMP_TIME_EXCEEDED		11
	#define ICMP_TTL_EXCEEDED	0
	#define ICMP_FRAG_EXCEEDED	1
#define ICMP_PARAMATER_PROB		12
#define ICMP_TIMESTAMP			13
#define ICMP_TIMESTAMP_REPLY	14
#define ICMP_INFO_REQUEST		15
#define ICMP_INFO_REPLY			16

//	be sure to take into account endianess and alignment issues 
//	if you use these structs to read and write to datagrams.
#pragma align=packed
// destination unreachable
// time exceeded
// parameter problem
// source quench
// redirect
struct icmp_msg1
{
	uint8		type;
	uint8		code;
	uint16		checksum;
	uint8		various[4];
	ip_header	ip_h;
	uint8		orig_datagram[64];
};

// echo
// info request
struct icmp_msg2
{
	uint8		type;
	uint8		code;
	uint16		checksum;
	uint16		identifier;
	uint16		seq_num;
};

struct icmp_timestamp
{
	uint8		type;
	uint8		code;
	uint16		checksum;
	uint16		identifier;
	uint16		seq_num;
	uint32		orig_timestamp;
	uint32		recv_timestamp;
	uint32		tran_timestamp;
};
#pragma align=reset

#endif