#ifndef _IP_H
#define _IP_H
/*
	$Author: me $ - $Date: 1999/03/21 04:04:01 $
*/
/*----------------------------------------------------------------------------------------
	NAT
------------------------------------------------------------------------------------------*/
// Network Address Translation for the BeOS
// Richard Burgess <rickb@spy.net>

/*----------------------------------------------------------------------------------------
	IP Header
------------------------------------------------------------------------------------------*/
//	RFC 791
//	be sure to take into account endianess and alignment issues 
//	if you use these structs to read and write to datagrams.
#pragma align=packed
struct ip_header 
{
#if B_HOST_IS_BENDIAN
	uint8	version:4,		// version
	 	  	header_length:4;// header length (in words)
#else
	uint8	header_length:4,// header length (in words)
			version:4;		// version
#endif
	uint8 	type;		// type of service
	uint16	length;		// total length
	uint16	id;			// identification
	uint16	offset;		// fragment offset field 
	uint8 	ttl;		// time to live
	uint8  	protocol;	// protocol
	uint16	checksum;	// checksum
	int32 	src;		// source ip address
	int32	dst;		// destination ip address
};
#pragma align=reset

// header offsets, our preferred way to access header elements
#define IP_LEN_VERSION_OFFSET	0
#define IP_TYPE_OFFSET			1
#define IP_LENGTH_OFFSET		2
#define IP_ID_OFFSET			4
#define IP_OFFSET_OFFSET		6
#define IP_TTL_OFFSET			8
#define IP_PROTOCOL_OFFSET		9
#define IP_CHECKSUM_OFFSET		10
#define IP_SRC_IP_OFFSET		12
#define IP_DST_IP_OFFSET		16

// accessors (on net-ordered data)
#define IP_VERSION(x) 		(x >> 4)
#define IP_HEADER_LENGTH(x)	( (x & 0xF) * 4) // in bytes

// protocol types
#define	IP_PROTO_IP		0	// dummy for IP 
#define	IP_PROTO_ICMP	1		// control message protocol 
#define	IP_PROTO_IGMP	2		// group mgmt protocol 
#define	IP_PROTO_GGP	3		// gateway^2 (deprecated) 
#define	IP_PROTO_IPIP	4		// IP inside IP 
#define	IP_PROTO_TCP	6		// tcp 
#define	IP_PROTO_EGP	8		// exterior gateway protocol 
#define	IP_PROTO_PUP	12		// pup 
#define	IP_PROTO_UDP	17		// user datagram protocol 
#define	IP_PROTO_IDP	22		// xns idp 
#define	IP_PROTO_TP		29 		// tp-4 w/ class negotiation 
#define	IP_PROTO_ESP	50 		// encap. security payload 
#define	IP_PROTO_AH		51 		// authentication header 
#define	IP_PROTO_EON	80		// ISO cnlp 
#define	IP_PROTO_ENCAP	98		// encapsulation header 

#define	IP_PROTO_RAW	255		// raw IP packet 
#define	IP_PROTO_MAX	256

#endif