#ifndef _NAT_H
#define _NAT_H
/*
	$Author: me $ - $Date: 1999/04/05 17:49:33 $
*/
/*----------------------------------------------------------------------------------------
	 NAT
------------------------------------------------------------------------------------------*/
// Network Address Translation for the BeOS
// Richard Burgess <rickb@spy.net>
#ifndef MSIPL_DEF_TEMPARG
#define MSIPL_DEF_TEMPARG
#endif
#include <map>
#include <list>

#include <NetProtocol.h>
#include <Locker.h>

#define NAT_NUM_PROTO	2

/*----------------------------------------------------------------------------------------
	 NATHandler
------------------------------------------------------------------------------------------*/
// RFC 1631
class NATHandler : public BNetProtocol, public BPacketHandler
{
public:
	NATHandler();
	virtual ~NATHandler();
	
	virtual void AddDevice(BNetDevice*, const char *name);
	virtual bool PacketReceived(BNetPacket*, BNetDevice*);
	
protected:
	void OutTranslateIpPacket(BNetPacket*);
	void OutTranslatePacket(BNetPacket*, uint32 orig_src_addr, uint32 new_src_addr, uint8 proto);
	
	void InTranslateIpPacket(BNetPacket*);
	uint32 InTranslatePacket(BNetPacket*, uint32 orig_dst_addr,  uint8 proto);
	
	uint32 GetInternetAddress();
	void FindPPPAddress(BNetPacket* pkt);
	
	void AddStaticPortMaps();
	
	void CheckFINFlag(BNetPacket*,uint16 port);
	
	// address mapping functions
	uint16 GetFreePort(uint8 proto);
	uint16 LookupPort(uint32 addr, uint16 port, uint8 proto);
	pair<uint32, uint16> LookupAddrPort(uint16 port, uint8 proto);

	// our local and internet devices and addresses	
	BNetDevice*		mLocalDev;
	uint32			mLocalAddr; // in network order
	uint32			mLocalMask; // in network order

	BNetDevice*		mInetDev;
	uint32			mInetAddr;  // in network order
	
	// lock, lock, lock
	BLocker	mPortMapLock[NAT_NUM_PROTO];
	BLocker mPortSocketLock[NAT_NUM_PROTO];
	BLocker mSocketLock[NAT_NUM_PROTO];
	
	// key = (real ip, real port), value = mapped port
	map<pair<uint32, uint16>, uint16> 	mPortMap[NAT_NUM_PROTO];
	
	// key = mapped port,  value = (real ip, real port),
	map<uint16, pair<uint32, uint16> > 	mReversePortMap[NAT_NUM_PROTO];
	
	// key = mapped port, value = (socket, time last used)
	map<uint16, pair<int, bigtime_t> > mPortSocketMap[NAT_NUM_PROTO];
	
	// pool of unused but allocated sockets (socket, port)
	list<pair<int, uint16> > mSocketPool[NAT_NUM_PROTO];
	
	// current port to bind new socket to
	int32 mFreePort[NAT_NUM_PROTO]; // host order

	// kills old sockets in mSocketPool
	thread_id	mSocketReaper; 	
	static int32 SocketReaper(void*);
	
	// kills old ports in mPortSocketMap (transfers sockets back to mSocketPool
	thread_id	mPortReaper; 	
	static int32 PortReaper(void*);
	
	// gets unused ports for us (puts in mSocketPool)
	thread_id	mTheGreatMaker;
	static int32 TheGreatMaker(void*);
};

#endif