#include <string.h>
#include <stdio.h>

#include <View.h>
#include <Window.h>
#include <MenuField.h>
#include <Menu.h>
#include <MenuItem.h>
#include <TextControl.h>
#include <ListView.h>
#include <ListItem.h>
#include <net_settings.h>
#include <netdb.h>
#include <Path.h>
#include <NetDevice.h>

#include "nat_config.h"

/*----------------------------------------------------------------------------------------
	 nat_config
------------------------------------------------------------------------------------------*/
nat_config::nat_config()
{
	nat_on = false;
	cur_lan_interface.heading[0] = '\0';
	cur_lan_interface.pretty_name[0] = '\0';
	
	cur_inet_interface.heading[0] = '\0';
	cur_inet_interface.pretty_name[0] = '\0';
	
	set_defaults();
}


nat_config::~nat_config()
{
}

/*----------------------------------------------------------------------------------------
	 set_defaults
------------------------------------------------------------------------------------------*/

void nat_config::set_defaults()
{
	start_port		= DFLT_START_PORT;
	end_port 		= DFLT_END_PORT;
	port_timeout 	= DFLT_PORT_TIMEOUT;
	socket_timeout 	= DFLT_SOCKET_TIMEOUT;
	socket_pool_size= DFLT_SOCKET_POOL_SIZE;
}

/*----------------------------------------------------------------------------------------
	 load_settings
------------------------------------------------------------------------------------------*/

void nat_config::load_settings()
{
	char pretty_name[255];
	
	find_net_setting(NULL,NAT_SETTINGS,LAN_INTERFACE,cur_lan_interface.heading,INTERFACE_LEN);
	get_pretty_name(cur_lan_interface.pretty_name, cur_lan_interface.heading);
	
	find_net_setting(NULL,NAT_SETTINGS,INET_INTERFACE,cur_inet_interface.heading,INTERFACE_LEN);
	get_pretty_name(cur_inet_interface.pretty_name, cur_inet_interface.heading);
	
	char str[SETTINGS_LEN];
	
	if(find_net_setting(NULL,NAT_SETTINGS,START_PORT,str,SETTINGS_LEN))
		start_port = atoi(str);
	
	if(find_net_setting(NULL,NAT_SETTINGS,END_PORT,str,SETTINGS_LEN))
			end_port = atoi(str);
	
	if(find_net_setting(NULL,NAT_SETTINGS,PORT_TIMEOUT,str,SETTINGS_LEN))
		port_timeout = atoi(str);
		
	if(find_net_setting(NULL,NAT_SETTINGS,SOCKET_TIMEOUT,str,SETTINGS_LEN))
		socket_timeout = atoi(str);
		
	if(find_net_setting(NULL,NAT_SETTINGS,SOCKET_POOL_SIZE,str,SETTINGS_LEN))
		socket_pool_size = atoi(str);

	// determine if nat is on or not
	nat_on = false;
	char protocols[255];
	if(find_net_setting(NULL,GLOBAL_SETTINGS,PROTOCOLS,protocols,255))
	{
		char* proto = strtok(protocols, " ");
		
		while(proto)
		{
			if(!strcmp(proto,NAT_SETTINGS))
			{
				nat_on = true;
				break;
			}
			proto = strtok(NULL, " ");	
		}
	}

	// load static port map info
	int count = 0;
	if(find_net_setting(NULL,NAT_SETTINGS,MAP_COUNT,str,SETTINGS_LEN))
		count = atoi(str);
		
	port_map pm;
	for(int i = 0; i < count; i++)
	{
		get_port_map(&pm,i);
		static_map.push_back(pm);
	}
}

/*----------------------------------------------------------------------------------------
	 save_settings
------------------------------------------------------------------------------------------*/

void nat_config::save_settings()
{	
	set_net_setting(NULL,NAT_SETTINGS,NULL,NULL);
	
	set_net_setting(NULL,NAT_SETTINGS,LAN_INTERFACE,cur_lan_interface.heading);
	set_net_setting(NULL,NAT_SETTINGS,INET_INTERFACE,cur_inet_interface.heading);
	
	char str[255];
	
	sprintf(str,"%hu",start_port);
	set_net_setting(NULL,NAT_SETTINGS,START_PORT,str);
	
	sprintf(str,"%hu",end_port);
	set_net_setting(NULL,NAT_SETTINGS,END_PORT,str);
	
	sprintf(str,"%d",port_timeout);
	set_net_setting(NULL,NAT_SETTINGS,PORT_TIMEOUT,str);
	
	sprintf(str,"%d",socket_timeout);
	set_net_setting(NULL,NAT_SETTINGS,SOCKET_TIMEOUT,str);
	
	sprintf(str,"%d",socket_pool_size);
	set_net_setting(NULL,NAT_SETTINGS,SOCKET_POOL_SIZE,str);
	
	char cur_protocols[255]; cur_protocols[0] = '\0';
	char protocols[255]; protocols[0] = '\0';
	
	// turn on ip forwarding
	if(nat_on)
	{
		set_net_setting(NULL,GLOBAL_SETTINGS,IP_FORWARD,"1");
		strcpy(protocols, NAT_SETTINGS);
	}
	
	// add to protocols line
	if(find_net_setting(NULL,GLOBAL_SETTINGS,PROTOCOLS,cur_protocols,255))
	{
		char* proto = strtok(cur_protocols, " ");
		
		while(proto)
		{	
			if(strcmp(proto,NAT_SETTINGS))
				sprintf(protocols,"%s %s", protocols, proto);
			proto = strtok(NULL, " ");	
		}
	}
	set_net_setting(NULL,GLOBAL_SETTINGS,PROTOCOLS,protocols);
	
	// save static port map info
	sprintf(str,"%d",(int)static_map.size());
	set_net_setting(NULL,NAT_SETTINGS,MAP_COUNT,str);

	int i =0;
	for(list<port_map>::iterator it = static_map.begin(); it != static_map.end(); it++, i++)
	{
		char str[INTERFACE_LEN];
		port_map_to_str(&(*it), str);
		
		char entry[INTERFACE_LEN];
		sprintf(entry,"%s%d",PORT_MAP,i);
		
		set_net_setting(NULL,NAT_SETTINGS,entry,str);
	}
}

/*----------------------------------------------------------------------------------------
	 configure_view
------------------------------------------------------------------------------------------*/

void nat_config::configure_view(BView* view)
{
	view->LockLooper();
	
	BMenuField* menuField = NULL;
	BMenu* menu = NULL;
	
	menuField = dynamic_cast<BMenuField*>(view->FindView(LAN_INTERFACE));
	if(menuField && ((menu = menuField->Menu()) != NULL) )
	{
		build_interface_menu(menu, lan_interfaces,LAN_INTERFACE_MSG);
		BMenuItem* item = menu->FindItem(cur_lan_interface.pretty_name);
		if(item) item->SetMarked(true);
	}
	
	menuField = dynamic_cast<BMenuField*>(view->FindView(INET_INTERFACE));
	if(menuField && ((menu = menuField->Menu()) != NULL) )
	{
		build_interface_menu(menu, inet_interfaces,INET_INTERFACE_MSG);
		BMenuItem* item = menu->FindItem(cur_inet_interface.pretty_name);
		if(item) item->SetMarked(true);
	}
	
	BTextControl* txtCtl = NULL;
	char str[255];
	
	txtCtl = dynamic_cast<BTextControl*>(view->FindView(START_PORT));
	if(txtCtl)
	{
		sprintf(str,"%hu",start_port);
		txtCtl->SetText(str);
	}
	
	txtCtl = dynamic_cast<BTextControl*>(view->FindView(END_PORT));
	if(txtCtl)
	{
		sprintf(str,"%hu",end_port);
		txtCtl->SetText(str);
	}
	
	txtCtl = dynamic_cast<BTextControl*>(view->FindView(PORT_TIMEOUT));
	if(txtCtl)
	{
		sprintf(str,"%d",port_timeout);
		txtCtl->SetText(str);
	}
	txtCtl = dynamic_cast<BTextControl*>(view->FindView(SOCKET_TIMEOUT));
	if(txtCtl)
	{
		sprintf(str,"%d",socket_timeout);
		txtCtl->SetText(str);
	}
	
	txtCtl = dynamic_cast<BTextControl*>(view->FindView(SOCKET_POOL_SIZE));
	if(txtCtl)
	{
		sprintf(str,"%d",socket_pool_size);
		txtCtl->SetText(str);	
	}
	
	BControl* ctl = dynamic_cast<BControl*>(view->FindView(NAT_SETTINGS));
	if(ctl)
		ctl->SetValue(nat_on);
	
	BListView* listView = dynamic_cast<BListView*>(view->FindView(PORT_MAP_LIST));
	if(listView)
	{
		listView->RemoveItems(0, listView->CountItems());
		listView->SetInvocationMessage(new BMessage(PORT_MAP_MSG));
		for(list<port_map>::iterator it = static_map.begin(); it != static_map.end(); it++)
		{
			port_map_to_str(&(*it),str);
			listView->AddItem(new BStringItem(str));
		}
	}
	
	view->UnlockLooper();	
}

/*----------------------------------------------------------------------------------------
	 get_from_view
------------------------------------------------------------------------------------------*/

void nat_config::get_from_view(BView* view)
{
	view->LockLooper();
	
	BMenuField* menuField = NULL;
	BMenu* menu = NULL;
	
	menuField = dynamic_cast<BMenuField*>(view->FindView(LAN_INTERFACE));
	if(menuField && ((menu = menuField->Menu()) != NULL) )
	{
		BMenuItem* item = menu->FindMarked();
		if(item)
		{
			strncpy(cur_lan_interface.pretty_name, item->Label(), 255);
			strncpy(cur_lan_interface.heading,item->Message()->FindString(WHICH_INTERFACE),INTERFACE_LEN);
		}
	}
	
	menuField = dynamic_cast<BMenuField*>(view->FindView(INET_INTERFACE));
	if(menuField && ((menu = menuField->Menu()) != NULL) )
	{
		BMenuItem* item = menu->FindMarked();
		if(item)
		{
			strncpy(cur_inet_interface.pretty_name, item->Label(), 255);
			strncpy(cur_inet_interface.heading,item->Message()->FindString(WHICH_INTERFACE),INTERFACE_LEN);
		}
	}
	
	BTextControl* txtCtl = NULL;
	
	txtCtl = dynamic_cast<BTextControl*>(view->FindView(START_PORT));
	if(txtCtl)
		start_port = atoi(txtCtl->Text());

	txtCtl = dynamic_cast<BTextControl*>(view->FindView(END_PORT));
	if(txtCtl)
		end_port = atoi(txtCtl->Text());

	txtCtl = dynamic_cast<BTextControl*>(view->FindView(PORT_TIMEOUT));
	if(txtCtl)
		port_timeout = atoi(txtCtl->Text());

	txtCtl = dynamic_cast<BTextControl*>(view->FindView(SOCKET_TIMEOUT));
	if(txtCtl)
		socket_timeout = atoi(txtCtl->Text());

	txtCtl = dynamic_cast<BTextControl*>(view->FindView(SOCKET_POOL_SIZE));
	if(txtCtl)
		socket_pool_size = atoi(txtCtl->Text());
	
	BControl* ctl = dynamic_cast<BControl*>(view->FindView(NAT_SETTINGS));
	if(ctl)
		nat_on = ctl->Value();
	
	BListView* listView = dynamic_cast<BListView*>(view->FindView(PORT_MAP_LIST));
	static_map.erase(static_map.begin(), static_map.end());
	if(listView)
	{
		for(int i = 0; ;i++)
		{
			BStringItem* item = dynamic_cast<BStringItem*>(listView->ItemAt(i));
			if(!item) break;
			
			port_map pm;
			str_to_port_map(&pm,item->Text());
			static_map.push_back(pm);
		}
	}
	
	view->UnlockLooper();	
}

/*----------------------------------------------------------------------------------------
	 make_interface_list
------------------------------------------------------------------------------------------*/
void nat_config::make_interface_list()
{
	build_interface_list(lan_interfaces,false);
	build_interface_list(inet_interfaces, true);
}

/*----------------------------------------------------------------------------------------
	 build_interface_list
------------------------------------------------------------------------------------------*/

void nat_config::build_interface_list(list<net_interface>& ni_list, bool inc_modem)
{
	ni_list.erase(ni_list.begin(), ni_list.end());
	
	char interfaces[255];

	if(find_net_setting(NULL,GLOBAL_SETTINGS,INTERFACES,interfaces,255))
	{
		char* interface = strtok(interfaces, " ");
		
		while(interface)
		{		
			net_interface ni;
	
			strncpy(ni.heading,interface,INTERFACE_LEN);
			get_pretty_name(ni.pretty_name, ni.heading);
				
			ni_list.push_back(ni);
			
			interface = strtok(NULL, " ");	
		}
	}
	
	if(inc_modem)
	{
		char modem[INTERFACE_LEN];
		char modem_type[255];	
		
		for(int i = 0; ; i++)
		{
			sprintf(modem,"%s%d",MODEM_INTERFACE,i);
			
			if(!find_net_setting(NULL,modem,MODEM_TYPE, modem_type, 255))
				break;
			else
			{
				net_interface ni;
	
				strncpy(ni.heading,modem,INTERFACE_LEN);
				get_pretty_name(ni.pretty_name, ni.heading);
							
				ni_list.push_back(ni);
			}
		}
	}
}

/*----------------------------------------------------------------------------------------
	 build_interface_menu
------------------------------------------------------------------------------------------*/

void nat_config::build_interface_menu(BMenu* menu, list<net_interface>& ni_list, uint32 cmd)
{
	int32 count = menu->CountItems();
	
	for(list<net_interface>::iterator it = ni_list.begin(); it != ni_list.end(); it++)
	{
		BMessage* msg = new BMessage(cmd);
		msg->AddString(WHICH_INTERFACE, (*it).heading);

		BMenuItem* item = new BMenuItem((*it).pretty_name,msg);
		menu->AddItem(item);
		
		item->SetMarked(true);
	}
	
	menu->RemoveItems(0,count,true);
}

/*----------------------------------------------------------------------------------------
	 get_port_map
------------------------------------------------------------------------------------------*/

void nat_config::get_port_map(port_map* pm,int idx)
{
	char str[INTERFACE_LEN];
	char range[255];
	sprintf(str,"%s%d",PORT_MAP, idx);
	if(find_net_setting( NULL, NAT_SETTINGS, str, range, INTERFACE_LEN))
		str_to_port_map(pm,range);
}

/*----------------------------------------------------------------------------------------
	 port_map_to_str
------------------------------------------------------------------------------------------*/

void nat_config::port_map_to_str(const port_map* pm, char* str)
{
	in_addr addr;
	addr.s_addr = pm->ip_addr;

	sprintf(str, "%hu-%hu:%s:%hu:%hu",pm->start_port,pm->end_port,inet_ntoa(addr),pm->local_start_port,pm->port_type);
}

/*----------------------------------------------------------------------------------------
	 str_to_port_map
------------------------------------------------------------------------------------------*/

void nat_config::str_to_port_map(port_map* pm, const char* in_str)
{
	char* str = strdup(in_str);
	char* tok = NULL;

	if((tok = strtok(str,"-")) != NULL)
		pm->start_port = atoi(tok);
		
	if((tok = strtok(NULL,":")) != NULL)
		pm->end_port = atoi(tok);
			
	if((tok = strtok(NULL,":")) != NULL)
		pm->ip_addr = inet_addr(tok);
			
	if((tok = strtok(NULL,":")) != NULL)
		pm->local_start_port = atoi(tok);
	
	if((tok = strtok(NULL," ")) != NULL)
		pm->port_type = atoi(tok);
	else
		pm->port_type = TCP_PORT | UDP_PORT;
		
	free(str);
}

/*----------------------------------------------------------------------------------------
	 get_pretty_name
------------------------------------------------------------------------------------------*/

void nat_config::get_pretty_name(char* pretty_name, const char* heading)
{
	char str[INTERFACE_LEN];
	if(!find_net_setting(NULL, heading, DEVICECONFIG, str, INTERFACE_LEN))
	{
		strncpy(pretty_name, heading, INTERFACE_LEN);
	}	
	else
	{
		if(!strcmp(str,"ppp"))
		{
			strcpy(pretty_name,"PPP");
		}else
		{
			if(find_net_setting(NULL, heading, IPADDRESS, str, INTERFACE_LEN))
				strncpy(pretty_name, str, INTERFACE_LEN);
			else
				strncpy(pretty_name, heading, INTERFACE_LEN);
			
	/*		// yeah, shouldn't hard code but all the net stuff is gonna change in R5 anyhow
			char path[255];
			sprintf(path,"/boot/beos/system/add-ons/net_server/%s",devconfig);
			image_id id= load_add_on(path);
			if(id > 0)
			{	
				BNetConfig* (*func)(const char*);
				if(get_image_symbol(id,"open_config",B_SYMBOL_TYPE_TEXT,(void**)&func) == B_OK)
				{
					BNetConfig* config = func("");	
					if(config) config->GetPrettyName(pretty_name,255);
				}
				unload_add_on(id);
			}*/
		}
	}	
}