#include <stdio.h>
#include <netdb.h>

#include <Application.h>
#include <View.h>
#include <Window.h>
#include <ListView.h>
#include <TextControl.h>
#include <MessageFilter.h>
#include <Archivable.h>
#include <Resources.h>

#include "nat_config.h"

nat_config config;
BView* config_view = NULL;

void make_config_view();
filter_result app_message_filter(BMessage*,BHandler**,BMessageFilter*);
void modify_port_map(BView*,BListView* listView, int32 idx);
void edit_port_map(BListView* listView, int32 idx);
void restart_networking();
void save_settings();
BArchivable* restore_from_resource(BResources* res , type_code type, int32 id);
void center_window(BWindow* win, BWindow* parent);

class NATApp : public BApplication
{
public:
	NATApp() : BApplication("application/x-vnd.rickb-natconfig") { make_config_view(); }
};

/*----------------------------------------------------------------------------------------
	 make_config_view
------------------------------------------------------------------------------------------*/

void make_config_view()
{
	BWindow* win = dynamic_cast<BWindow*>(restore_from_resource(be_app->AppResources(), ARCHIVE_RES_TYPE, CONFIG_WINDOW_ID));
	
	if(win)
		config_view = win->FindView(THE_VIEW);
	
	if(!config_view)
	{
		printf("error loading resources\n");
		be_app->PostMessage(B_QUIT_REQUESTED);
	}else
	{					
// have to set up list view manually since i can't do it in beconstruct
		BMessage* msg = NULL;
		BInvoker* ctl = NULL;
		BListView* listView = NULL;
		BView* view = config_view->FindView(STATIC_MAP_LIST);
		
		if(view)
		{
			BRect frame(0,0,195,60);
			frame.OffsetTo(10,15);
			listView = new BListView(frame,PORT_MAP_LIST);
			view->AddChild(listView);
		}
		
		ctl = dynamic_cast<BInvoker*>(config_view->FindView(ADD_BUTTON));
		if(ctl && ((msg = ctl->Message()) != NULL))
			msg->AddPointer("source",(void*)listView);
		
		ctl = dynamic_cast<BInvoker*>(config_view->FindView(DELETE_BUTTON));
		if(ctl && ((msg = ctl->Message()) != NULL))
			msg->AddPointer("source",(void*)listView);
			
		config.make_interface_list();
		config.load_settings();
		config.configure_view(config_view);

		win->AddCommonFilter(new BMessageFilter(B_ANY_DELIVERY, B_ANY_SOURCE,app_message_filter));
		win->SetWorkspaces(B_CURRENT_WORKSPACE);
		win->Show();
	}
}

/*----------------------------------------------------------------------------------------
	 app_message_filter
------------------------------------------------------------------------------------------*/

filter_result app_message_filter(BMessage* msg, BHandler** handler, BMessageFilter* filter)
{
	filter_result result = B_SKIP_MESSAGE;
	
	switch(msg->what)
	{
		case OK_MSG:
			save_settings();
			be_app->PostMessage(B_QUIT_REQUESTED);
		break;
		
		case RESTART_NET_MSG:
			save_settings();
			restart_networking();
		break;
		
		case CANCEL_MSG:
			be_app->PostMessage(B_QUIT_REQUESTED);
		break;
		
		case DEFAULTS_MSG:
			config.set_defaults();
			if(config_view)
				config.configure_view(config_view);
		break;
		
		case PORT_MAP_MSG:
		case ADD_PORT_MAP:
		{	
			BListView* listView = NULL;
			int32 idx = 0;
			
			msg->FindPointer("source",(void**)&listView);
			if(msg->FindInt32("index",&idx) != B_OK)
				idx = listView->CountItems() + 1;
				
			if(listView)
				edit_port_map(listView, idx);
		}
		break;
		
		break;
		
		case DEL_PORT_MAP:
		{
			BListView* listView = NULL;
			
			msg->FindPointer("source",(void**)&listView);
			if(listView)
				delete listView->RemoveItem(listView->CurrentSelection());
		}
		break;
		
		case PORT_CANCEL_MSG:
			filter->Looper()->PostMessage(B_QUIT_REQUESTED);
		break;

		case PORT_OK_MSG:
		{
			int32 idx = msg->FindInt32(MAP_IDX);
				
			BListView* listView = NULL;
			msg->FindPointer(LIST_VIEW,(void**)&listView);
			BWindow* win = dynamic_cast<BWindow*>(*handler);
			BView* view = NULL;
			
			if(win) 
				view = win->FindView(THE_VIEW);
				
			if(view && listView)
				modify_port_map(view, listView, idx);
				
			win->PostMessage(B_QUIT_REQUESTED);				
		}
		break;
		
		default:
			result = B_DISPATCH_MESSAGE;
		break;
	}

	return result;
}
/*----------------------------------------------------------------------------------------
	 modify_port_map
------------------------------------------------------------------------------------------*/

void modify_port_map(BView* view, BListView* listView, int32 idx)
{
	BStringItem* item = dynamic_cast<BStringItem*>(listView->ItemAt(idx));
	
	if(!item)
	{
		item = new BStringItem("");
		listView->LockLooper();
		listView->AddItem(item);
		listView->UnlockLooper();
	}
		
	if(item)
	{
		config_view->LockLooper();
		
		port_map pm;
		BTextControl* txtCtl = NULL;
		
		txtCtl = dynamic_cast<BTextControl*>(view->FindView(IN_START_PORT));
		if(txtCtl)
			pm.start_port = atoi(txtCtl->Text());
		
		txtCtl = dynamic_cast<BTextControl*>(view->FindView(IN_END_PORT));
		if(txtCtl)
			pm.end_port = atoi(txtCtl->Text());
		else
			pm.end_port = pm.start_port;
			
		txtCtl = dynamic_cast<BTextControl*>(view->FindView(IP_ADDR));
		if(txtCtl)
			pm.ip_addr = inet_addr(txtCtl->Text());

		txtCtl = dynamic_cast<BTextControl*>(view->FindView(LOCAL_START_PORT));
		if(txtCtl)
			pm.local_start_port = atoi(txtCtl->Text());
		else
			pm.local_start_port = pm.start_port;
		
		pm.port_type = TCP_PORT | UDP_PORT;
		
		config_view->UnlockLooper();

		char str[255];
		config.port_map_to_str(&pm,str);

		item->SetText(str);
		listView->LockLooper();
		listView->Invalidate();
		listView->UnlockLooper();
	}
}

/*----------------------------------------------------------------------------------------
	 edit_port_map
------------------------------------------------------------------------------------------*/

void edit_port_map(BListView* listView, int32 idx)
{	
	BWindow* win = dynamic_cast<BWindow*>(restore_from_resource(be_app->AppResources(), ARCHIVE_RES_TYPE, PORT_MAP_DIALOG_ID));
	if(!win) return;
	win->SetFeel(B_MODAL_APP_WINDOW_FEEL);
	
	BStringItem* item = dynamic_cast<BStringItem*>(listView->ItemAt(idx));
	port_map pm;
	if(item)
		config.str_to_port_map(&pm, item->Text());
	else
		pm.start_port = pm.end_port = pm.ip_addr = pm.local_start_port = 0;
	
	BTextControl* txtCtl = NULL;
	char str[SETTINGS_LEN];
	
	txtCtl = dynamic_cast<BTextControl*>(win->FindView(IN_START_PORT));
	if(txtCtl)
	{
		sprintf(str,"%d",pm.start_port);
		txtCtl->SetText(str);
	}
	
	txtCtl = dynamic_cast<BTextControl*>(win->FindView(IN_END_PORT));
	if(txtCtl)
	{
		sprintf(str, "%d",pm.end_port);
		txtCtl->SetText(str);
	}				
	
	txtCtl = dynamic_cast<BTextControl*>(win->FindView(IP_ADDR));
	if(txtCtl)
	{	
		in_addr addr;
		addr.s_addr = pm.ip_addr;
	
		txtCtl->SetText(inet_ntoa(addr));
	}
	
	txtCtl = dynamic_cast<BTextControl*>(win->FindView(LOCAL_START_PORT));
	if(txtCtl)
	{
		sprintf(str, "%d",pm.local_start_port);
		txtCtl->SetText(str);
	}				
		
	BInvoker* ctl = dynamic_cast<BInvoker*>(win->FindView(OK_BUTTON));
	if(ctl)
	{
		BMessage* msg = ctl->Message();
		if(msg)
		{
			msg->AddInt32(MAP_IDX, idx);
			msg->AddPointer(LIST_VIEW,(void*)listView);
		}
		
	}
	
	win->AddCommonFilter(new BMessageFilter(B_ANY_DELIVERY, B_ANY_SOURCE,app_message_filter));
	
	center_window(win, config_view->Window());
	win->Show();	
}


/*----------------------------------------------------------------------------------------
	 restart_networking
------------------------------------------------------------------------------------------*/

void restart_networking()
{
	BMessenger msgr("application/x-vnd.Be-NETS");
	
	BMessage reply;
	msgr.SendMessage(B_QUIT_REQUESTED);

	snooze(5 * 1000000);
	
	system("/boot/beos/system/boot/Netscript");
}

/*----------------------------------------------------------------------------------------
	 save_settings
------------------------------------------------------------------------------------------*/

void save_settings()
{
	if(config_view)
	{
		config.get_from_view(config_view);
		config.save_settings();
	}
}

/*----------------------------------------------------------------------------------------
	 restore_from_resource
------------------------------------------------------------------------------------------*/

BArchivable* restore_from_resource(BResources* res , type_code type, int32 id)
{
	BMessage msg;
		
	uint32 len = 0;
	const char* data = NULL;
	
	data = (const char*)res->FindResource(type, id, &len);
	
	if(data)
		msg.Unflatten(data);
	
	return instantiate_object(&msg);
}

/*----------------------------------------------------------------------------------------
	 center_window
------------------------------------------------------------------------------------------*/
void center_window(BWindow* win, BWindow* parent)
{
	BRect bounds = win->Bounds();
	BRect parent_frame = parent->Frame();
	
	float x = (parent_frame.Width() - bounds.Width()) / 2;
	float y = (parent_frame.Height() - bounds.Height()) / 2;

	win->MoveTo(parent_frame.left + x, parent_frame.top + y);
}

/*----------------------------------------------------------------------------------------
	 main
------------------------------------------------------------------------------------------*/

int main(int, char**)
{
	(new NATApp())->Run();
	
	return 0;
}